@@ -268,12 +268,13 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
         }
     }
 
-    ref<YIcon> icon(getFrame()->getIcon());
+    ref<YIcon> icon;
     bool iconDrawn = false;
-
-    if (taskBarShowWindowIcons && icon != null) {
+    if (taskBarShowWindowIcons) {
+        icon = getFrame()->getIcon();
+    }
+    if (icon != null) {
         int iconSize = YIcon::smallSize();
-
         int const y((height() - 3 - iconSize -
                      (wmLook == lookMetal)) / 2);
         iconDrawn = icon->draw(g, p + max(1, left), p + 1 + y, iconSize);
@@ -286,9 +287,8 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
     }
 
     mstring str = getFrame()->getIconTitle();
-    if (str == null || str.length() == 0)
+    if (str.isEmpty())
         str = getFrame()->getTitle();
-
     if (str != null) {
         ref<YFont> font = getFont();
         if (font != null) {
@@ -297,7 +297,7 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
 
             int iconSize = 0;
             int pad = max(1, left);
-            if (taskBarShowWindowIcons && iconDrawn) {
+            if (iconDrawn) {
                 iconSize = YIcon::smallSize();
                 pad += 2;
             }
@@ -502,10 +502,6 @@ void TaskPane::insert(TaskBarApp *tapp) {
     (--it).insert(tapp);
 }
 
-void TaskPane::remove(TaskBarApp *tapp) {
-    findRemove(fApps, tapp);
-}
-
 TaskBarApp* TaskPane::predecessor(TaskBarApp *tapp) {
     const int count = fApps.getCount();
     const int found = find(fApps, tapp);
@@ -552,13 +548,9 @@ TaskBarApp *TaskPane::addApp(YFrameWindow *frame) {
     return tapp;
 }
 
-void TaskPane::removeApp(YFrameWindow *frame) {
-    TaskBarApp* task = findApp(frame);
-    if (task) {
-        task->hide();
-        remove(task);
-        relayout();
-    }
+void TaskPane::remove(TaskBarApp* task) {
+    task->setShown(false);
+    findRemove(fApps, task);
 }
 
 void TaskPane::relayout(bool force) {
