@@ -10,32 +10,26 @@
 
 #ifdef CONFIG_SESSION
 
-#include "yfull.h"
 #include "wmframe.h"
 #include "wmsession.h"
-#include "base.h"
 #include "wmapp.h"
-
-#include <stdio.h>
-#include <string.h>
+#include "workspaces.h"
 
 #include "intl.h"
 
-SMWindowKey::SMWindowKey(YFrameWindow */*f*/) {
+SMWindowKey::SMWindowKey(YFrameWindow *):
+    clientId(null), windowRole(null), windowClass(null), windowInstance(null)
+{
 }
 
-SMWindowKey::SMWindowKey(char *id, char *role) {
-    clientId = newstr(id);
-    windowRole = newstr(role);
-    windowClass = 0;
-    windowInstance = 0;
+SMWindowKey::SMWindowKey(mstring id, mstring role):
+    clientId(id), windowRole(role), windowClass(null), windowInstance(null)
+{
 }
 
-SMWindowKey::SMWindowKey(char *id, char *klass, char *instance) {
-    clientId = newstr(id);
-    windowRole = 0;
-    windowClass = newstr(klass);
-    windowInstance = newstr(instance);
+SMWindowKey::SMWindowKey(mstring id, mstring klass, mstring instance):
+    clientId(id), windowRole(null), windowClass(klass), windowInstance(instance)
+{
 }
 
 SMWindowKey::~SMWindowKey() {
@@ -44,7 +38,7 @@ SMWindowKey::~SMWindowKey() {
 SMWindowInfo::SMWindowInfo(YFrameWindow *f): key(f) {
 }
 
-SMWindowInfo::SMWindowInfo(char *id, char *role,
+SMWindowInfo::SMWindowInfo(mstring id, mstring role,
                            int ax, int ay, int w, int h,
                            unsigned long astate, int alayer, int aworkspace): key(id, role)
 {
@@ -57,7 +51,7 @@ SMWindowInfo::SMWindowInfo(char *id, char *role,
     workspace = aworkspace;
 }
 
-SMWindowInfo::SMWindowInfo(char *id, char *klass, char *instance,
+SMWindowInfo::SMWindowInfo(mstring id, mstring klass, mstring instance,
                            int ax, int ay, int w, int h,
                            unsigned long astate, int alayer, int aworkspace): key(id, klass, instance)
 {
@@ -77,10 +71,10 @@ void SMWindows::addWindowInfo(SMWindowInfo *info) {
     fWindows.append(info);
 }
 
-void SMWindows::setWindowInfo(YFrameWindow */*f*/) {
+void SMWindows::setWindowInfo(YFrameWindow *) {
 }
-                                     
-bool SMWindows::getWindowInfo(YFrameWindow */*f*/, SMWindowInfo */*info*/) {
+
+bool SMWindows::getWindowInfo(YFrameWindow *, SMWindowInfo *) {
     return false;
 }
 
@@ -89,18 +83,18 @@ bool SMWindows::findWindowInfo(YFrameWindow *f) {
     Window leader = f->client()->clientLeader();
     if (leader == None) return false;
 
-    char *cid = f->client()->getClientId(leader);
-    if (cid == 0) return false;
+    mstring cid = f->client()->getClientId(leader);
+    if (cid == null) return false;
 
-    for (unsigned i = 0; i < fWindows.getCount(); ++i) {
+    for (int i = 0; i < fWindows.getCount(); ++i) {
         const SMWindowInfo *window = fWindows.getItem(i);
 
-        if (strcmp(cid, window->key.clientId) == 0) {
-            if (window->key.windowClass &&
-                window->key.windowInstance)
+        if (cid.equals(window->key.clientId)) {
+            if (window->key.windowClass != null &&
+                window->key.windowInstance != null)
             {
-                char *klass = 0;
-                char *instance = 0;
+                mstring klass = null;
+                mstring instance = null;
                 XClassHint *ch = f->client()->classHint();
 
                 if (ch) {
@@ -108,10 +102,11 @@ bool SMWindows::findWindowInfo(YFrameWindow *f) {
                     instance = ch->res_name;
                 }
 
-                if (strcmp(klass, window->key.windowClass) == 0 &&
-                    strcmp(instance, window->key.windowInstance) == 0) {
-                    MSG(("got c %s %s %s %d:%d:%d:%d %d %ld %d", 
-                         cid, klass, instance,
+                if (klass.equals(window->key.windowClass) &&
+                    instance.equals(window->key.windowInstance))
+                {
+                    MSG(("got c %s %s %s %d:%d:%d:%d %d %ld %d",
+                         cid.c_str(), klass.c_str(), instance.c_str(),
                          window->x, window->y, window->width, window->height,
                          window->workspace, window->state, window->layer));
                     f->configureClient(window->x, window->y,
@@ -119,23 +114,19 @@ bool SMWindows::findWindowInfo(YFrameWindow *f) {
                     f->setRequestedLayer(window->layer);
                     f->setWorkspace(window->workspace);
                     f->setState(WIN_STATE_ALL, window->state);
-
-                    XFree(cid);
                     return true;
                 }
             }
         }
     }
-
-    XFree(cid);
     return false;
 }
 
-bool SMWindows::removeWindowInfo(YFrameWindow */*f*/) {
+bool SMWindows::removeWindowInfo(YFrameWindow *) {
     return false;
 }
 
-SMWindows *sminfo = 0;
+SMWindows *sminfo = nullptr;
 
 static int wr_str(FILE *f, const char *s) {
     if (!s)
@@ -169,33 +160,24 @@ static int wr_str(FILE *f, const char *s) {
 }
 
 static int rd_str(char *s, char *d) {
-    char c;
-    bool old = true;
+    while (*s == ' ')
+        ++s;
 
-    c = *s++;
-    while (c == ' ')
-        c = *s++;
-    if (c == '"') {
-        old = false;
-        c = *s++;
-    }
+    bool quoted = (*s == '"');
+    if (quoted)
+        ++s;
 
-    while (c != 0) {
-        if (c == '"' && !old) {
-            c = *s++;
+    for (char c = *s; c; c = *s++) {
+        if (quoted ? c == '"' : c == ' ')
             break;
-        }
-        if (c == ' ' && old)
-            break;
-        if (!old && c == '=') {
-            int i = ' ';
-
-            sscanf(s, "%02X", &i);
-            s += 2;
-            c = (char)(i & 0xFF);
+        if (quoted && c == '=') {
+            unsigned int i = ' ';
+            if (sscanf(s, "%02X", &i) > 0) {
+                s += 2;
+                c = (char)(i & 0xFF);
+            }
         }
         *d++ = c;
-        c = *s++;
     }
     *d = 0;
     return 0;
@@ -204,7 +186,6 @@ static int rd_str(char *s, char *d) {
 void loadWindowInfo() {
     sminfo = new SMWindows();
 
-    FILE *fp;
     char line[1024];
     char cid[1024];
     char role[1024];
@@ -215,16 +196,12 @@ void loadWindowInfo() {
     unsigned long state;
     SMWindowInfo *info;
 
-    char *name = getsesfile();
-
-    if (name == 0)
+    upath name = getsesfile();
+    FILE *fp = name.fopen("r");
+    if (fp == nullptr)
         return ;
 
-    fp = fopen(name, "r");
-    if (fp == NULL)
-        return ;
-
-    while (fgets(line, sizeof(line), fp) != 0) {
+    while (fgets(line, sizeof(line), fp) != nullptr) {
         if (line[0] == 'c') {
             if (sscanf(line, "c %s %s %s %d:%d:%d:%d %ld %lu %ld",
                        cid, klass, instance, &x, &y, &w, &h,
@@ -261,8 +238,8 @@ void loadWindowInfo() {
         } else if (line[0] == 'w') {
             int ws = 0;
 
-            if (sscanf(line, "w %d", &ws) == 1) {        
-                if (ws >= 0 && ws < manager->workspaceCount())
+            if (sscanf(line, "w %d", &ws) == 1) {
+                if (ws >= 0 && ws < workspaceCount)
                     manager->activateWorkspace(ws);
             }
         } else {
@@ -285,15 +262,10 @@ void YWMApp::smDie() {
 }
 
 void YWMApp::smSaveYourselfPhase2() {
-    FILE *fp;
-    char *name = getsesfile();
-    YFrameWindow *f = 0;
-    
-    if (name == 0)
-        goto end;
-
-    fp = fopen(name, "w+");
-    if (fp == NULL)
+    upath name = getsesfile();
+    YFrameWindow *f = nullptr;
+    FILE *fp = name.fopen("w+");
+    if (fp == nullptr)
         goto end;
 
     f = manager->topLayer();
@@ -305,23 +277,21 @@ void YWMApp::smSaveYourselfPhase2() {
         //msg("window=%s", f->client()->windowTitle());
         if (leader != None) {
             //msg("leader=%lX", leader);
-            char *cid = 0;
+            mstring cid = f->client()->getClientId(leader);
 
-            cid = f->client()->getClientId(leader);
-
-            if (cid) {
+            if (cid != null) {
                 f->client()->getWindowRole();
-                const char *role = f->client()->windowRole();
+                mstring role = f->client()->windowRole();
 
-                if (role) {
+                if (role != null) {
                     fprintf(fp, "r ");
                     //%s %s ", cid, role);
-                    wr_str(fp, cid);
-                    wr_str(fp, role);
+                    wr_str(fp, cid.c_str());
+                    wr_str(fp, role.c_str());
                 } else {
                     f->client()->getClassHint();
-                    char *klass = 0;
-                    char *instance = 0;
+                    char *klass = nullptr;
+                    char *instance = nullptr;
                     XClassHint *ch = f->client()->classHint();
                     if (ch) {
                         klass = ch->res_class;
@@ -332,16 +302,14 @@ void YWMApp::smSaveYourselfPhase2() {
                         //msg("k=%s, i=%s", klass, instance);
                         fprintf(fp, "c ");
                         //%s %s %s ", cid, klass, instance);
-                        wr_str(fp, cid);
+                        wr_str(fp, cid.c_str());
                         wr_str(fp, klass);
                         wr_str(fp, instance);
                     } else {
-                        XFree(cid);
                         continue;
                     }
                 }
-                XFree(cid);
-                fprintf(fp, "%d:%d:%d:%d %ld %lu %ld\n",
+                fprintf(fp, "%d:%d:%d:%d %d %lu %ld\n",
                         f->x(), f->y(), f->client()->width(), f->client()->height(),
                         f->getWorkspace(), f->getState(), f->getActiveLayer());
             }
@@ -354,3 +322,5 @@ end:
 }
 
 #endif /* CONFIG_SESSION */
+
+// vim: set sw=4 ts=4 et:
